@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright 2013 Brocade Communications System, Inc.
+# Copyright 2013 Brocade Communications System,  Inc.
 # All rights reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -41,6 +41,7 @@ from quantum.db import api as db
 from quantum.db import db_base_plugin_v2
 from quantum.db import dhcp_rpc_base
 from quantum.db import l3_db
+from quantum.db import extraroute_db
 from quantum.db import l3_rpc_base
 from quantum.db import securitygroups_rpc_base as sg_db_rpc
 from quantum.extensions import portbindings
@@ -198,6 +199,7 @@ class AgentNotifierApi(proxy.RpcProxy,
 
 
 class BrocadePluginV2(db_base_plugin_v2.QuantumDbPluginV2,
+                      extraroute_db.ExtraRoute_db_mixin,
                       sg_db_rpc.SecurityGroupServerRpcMixin,
                       agentschedulers_db.AgentSchedulerDbMixin):
     """BrocadePluginV2 is a Quantum plugin.
@@ -213,6 +215,7 @@ class BrocadePluginV2(db_base_plugin_v2.QuantumDbPluginV2,
         """
 
         self.supported_extension_aliases = ["binding", "security-group",
+                                            "router", "extraroute",
                                             "agent", "agent_scheduler"]
         self.binding_view = "extension:port_binding:view"
         self.binding_set = "extension:port_binding:set"
@@ -270,16 +273,17 @@ class BrocadePluginV2(db_base_plugin_v2.QuantumDbPluginV2,
                                             switch['username'],
                                             switch['password'],
                                             vlan_id)
-            except Exception as e:
+            except Exception:
                 # Proper formatting
-                LOG.warning(_("Brocade NOS driver:"))
-                LOG.warning(_("%s"), e)
+                LOG.exception(_("Brocade NOS driver error"))
                 LOG.debug(_("Returning the allocated vlan (%d) to the pool"),
                           vlan_id)
                 self._vlan_bitmap.release_vlan(int(vlan_id))
                 raise Exception("Brocade plugin raised exception, check logs")
 
             brocade_db.create_network(context, net_uuid, vlan_id)
+            self._process_l3_create(context, network['network'], net['id'])
+            self._extend_network_dict_l3(context, net)
 
         LOG.info(_("Allocated vlan (%d) from the pool"), vlan_id)
         return net
@@ -309,12 +313,12 @@ class BrocadePluginV2(db_base_plugin_v2.QuantumDbPluginV2,
                 self._driver.delete_network(switch['address'],
                                             switch['username'],
                                             switch['password'],
-                                            net_id)
-            except Exception as e:
+                                            vlan_id)
+            except Exception:
                 # Proper formatting
-                LOG.warning(_("Brocade NOS driver:"))
-                LOG.warning(_("%s"), e)
-                raise Exception("Brocade plugin raised exception, check logs")
+                LOG.exception(_("Brocade NOS driver error"))
+                raise Exception(_("Brocade plugin raised exception, "
+                                  "check logs"))
 
             # now ok to delete the network
             brocade_db.delete_network(context, net_id)
@@ -323,6 +327,37 @@ class BrocadePluginV2(db_base_plugin_v2.QuantumDbPluginV2,
         self._vlan_bitmap.release_vlan(int(vlan_id))
         return result
 
+    def update_network(self, context, id, network):
+
+        session = context.session
+        with session.begin(subtransactions=True):
+            net = super(BrocadePluginV2, self).update_network(context, id,
+                                                              network)
+            self._process_l3_update(context, network['network'], id)
+            self._extend_network_dict_l3(context, net)
+        return net
+
+    def get_network(self, context, id, fields=None):
+        session = context.session
+        with session.begin(subtransactions=True):
+            net = super(BrocadePluginV2, self).get_network(context,
+                                                           id, None)
+            self._extend_network_dict_l3(context, net)
+
+        return self._fields(net, fields)
+
+    def get_networks(self, context, filters=None, fields=None,
+                     sorts=None, limit=None, marker=None, page_reverse=False):
+        session = context.session
+        with session.begin(subtransactions=True):
+            nets = super(BrocadePluginV2,
+                         self).get_networks(context, filters, None, sorts,
+                                            limit, marker, page_reverse)
+            for net in nets:
+                self._extend_network_dict_l3(context, net)
+
+        return [self._fields(net, fields) for net in nets]
+
     def create_port(self, context, port):
         """Create logical port on the switch."""
 
@@ -351,11 +386,11 @@ class BrocadePluginV2(db_base_plugin_v2.QuantumDbPluginV2,
                                                       switch['password'],
                                                       vlan_id,
                                                       mac)
-            except Exception as e:
+            except Exception:
                 # Proper formatting
-                LOG.warning(_("Brocade NOS driver:"))
-                LOG.warning(_("%s"), e)
-                raise Exception("Brocade plugin raised exception, check logs")
+                LOG.exception(_("Brocade NOS driver error"))
+                raise Exception(_("Brocade plugin raised exception, "
+                                  "check logs"))
 
             # save to brocade persistent db
             brocade_db.create_port(context, port_id, network_id,
@@ -367,6 +402,26 @@ class BrocadePluginV2(db_base_plugin_v2.QuantumDbPluginV2,
 
     def delete_port(self, context, port_id):
         with context.session.begin(subtransactions=True):
+            neutron_port = self.get_port(context, port_id)
+            interface_mac = neutron_port['mac_address']
+            # convert mac format: xx:xx:xx:xx:xx:xx -> xxxx.xxxx.xxxx
+            mac = self.mac_reformat_62to34(interface_mac)
+
+            brocade_port = brocade_db.get_port(context, port_id)
+            vlan_id = brocade_port['vlan_id']
+
+            switch = self._switch
+            try:
+                self._driver.dissociate_mac_from_network(switch['address'],
+                                                         switch['username'],
+                                                         switch['password'],
+                                                         vlan_id,
+                                                         mac)
+            except Exception:
+                LOG.exception(_("Brocade NOS driver error"))
+                raise Exception(
+                    _("Brocade plugin raised exception, check logs"))
+
             super(BrocadePluginV2, self).delete_port(context, port_id)
             brocade_db.delete_port(context, port_id)
 
